#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>

#include <rtai_shm.h>
#include <rtai_nam2num.h>

#include <math.h>
#include "freqresp.h"

static float * data;
static struct my_msg_struct * msg;

extern void realfft(double *, unsigned int, double *);

static void
emergency_stop(int d)
{
        msg->command = STOP_TASK;
	rtai_free(nam2num("data"), data);
	rtai_free(nam2num("msg"), msg);

	exit(1);
}

int
main(int argc, char *argv[])
{
	FILE *fp[3]; // fp[0]:SPK1 $BMQ(B, fp[1]:SPK2 $BMQ(B, fp[2]:SPK3 $BMQ(B
	int j, l,repeat,count;
	static double u_time[LENGTH_MAX]; // $B@)8fF~NO$N;~4VGH7A(B
	static double y0_time[LENGTH_MAX]; // $B%(%i!<%^%$%/=PNO$N;~4VGH7A(B
	static double y1_time[LENGTH_MAX]; // $B%j%U%!%l%s%9%^%$%/=PNO$N;~4VGH7A(B
	static double u_fft[LENGTH_MAX], y0_fft[LENGTH_MAX], y1_fft[LENGTH_MAX];
	double amp[3] = {AMP_CH0, AMP_CH1, AMP_CH2};
	double f, f10;
	double u_a, u_b, y0_a, y0_b, y1_a, y1_b;
	double y0_u_gain, y0_u_phase, y1_u_gain, y1_u_phase;
	float t;
	int div;
	double T, Ts;

	fp[0] = fopen("data/spk1.dat", "w"); //
	fp[1] = fopen("data/spk2.dat", "w"); //
	fp[2] = fopen("data/spk3.dat", "w"); //

	if((data = (float *)rtai_malloc(nam2num("data"), 0)) == NULL){
		fprintf(stderr, "rtai_malloc() failed!!!\n");
		return -1;
	}

	if((msg = (struct my_msg_struct *)rtai_malloc(nam2num("msg"), 0)) == NULL){
		fprintf(stderr, "rtai_malloc failed \n");
		return -1;
	}

	msg->command = STOP_TASK;

	signal(SIGINT, emergency_stop);

	for(msg->spk_num = 0; msg->spk_num <= 0; msg->spk_num++){
	//for(msg->spk_num = 0; msg->spk_num <= 1; msg->spk_num++){
		
		msg->amp = amp[msg->spk_num];

		f10 = F10_LOW;
#ifndef DEBUG
		while(f10 < F10_HIGH){
#endif
			msg->sampling_period= SAMPLING_PERIOD;
			f = pow(10.0, f10);
				
			T = 1. / f;
			msg->resolution = T / msg->sampling_period + 1; // 10Hz $B;~!$(B1999 $B$K$J$C$F$7$^$&$N$r(B 2000 $B$KJd@5!%(B
			msg->sampling_period  = T / (double)msg->resolution;

			repeat = LENGTH_MAX / msg->resolution; // LENGTH_MAX $B$N%G!<%?D9$G!$@589GH$,2?8D5-O?$G$-$k$+(B?
			msg->recording_samples = repeat * msg->resolution;
			data[BUF_LEN * (msg->recording_samples - 1)] = -1.; /* $B;~4V$r3JG<$9$kG[Ns$N:G8e$K!$(B-1 $B$r%U%i%0$H$7$FF~$l$F$*$/(B */
					
			msg->canceling_samples = WAITING_TIME / msg->sampling_period; /* $B5-O?$;$:$K<N$F$k%5%s%W%k$NE@?t(B */
			fprintf(stderr, "sampling_period=%.8f, resolution=%d, recording_samples=%d, canceling_samples=%d\n", msg->sampling_period, msg->resolution, msg->recording_samples, msg->canceling_samples);
			
			/* now start the tasks */
			fprintf(stderr, "Starting real time task ... ");
			msg->command = START_TASK;
			fprintf(stderr, "done.\n");
				
			fprintf(stderr, "Waiting for record ... ");
			while (data[BUF_LEN * (msg->recording_samples - 1)] < 0.){
			  fprintf(stderr, "\r");
			} /* $B@5>o$K%G!<%?$,=q$-$3$^$l$l$P!$%U%i%0$O@5$K$J$k$O$:(B */
			fprintf(stderr, "done.\n");

			j = 0;
			count = 0;
			while(count < BUF_LEN * msg->recording_samples){
				t = data[count++]; /* $B%@%_!<(B */
				u_time[j] = data[count++];
				y0_time[j] = data[count++];
				y1_time[j] = data[count++];
#ifdef DEBUG			
				fprintf(stdout, "%g %g %g %g\n", t, u_time[j], y0_time[j], y1_time[j]);
#endif
				j++;
			}

#ifndef DEBUG
			realfft(u_time, msg->recording_samples, u_fft);
			u_b = u_fft[repeat * 2 - 1];
			u_a = u_fft[repeat * 2];

			realfft(y0_time, msg->recording_samples, y0_fft);
			y0_b = y0_fft[repeat * 2 - 1];
			y0_a = y0_fft[repeat * 2];
					
			realfft(y1_time, msg->recording_samples, y1_fft);
			y1_b = y1_fft[repeat * 2 - 1];
			y1_a = y1_fft[repeat * 2];
					
			y0_u_gain = sqrt(y0_a * y0_a + y0_b * y0_b) / sqrt(u_a * u_a + u_b * u_b);
			y0_u_phase = (((atan(y0_b / y0_a) + M_PI * (y0_a < 0)) -(atan(u_b / u_a) + M_PI * (u_a < 0))) * 180.0 / M_PI);

			y1_u_gain = sqrt(y1_a * y1_a + y1_b * y1_b) / sqrt(u_a * u_a + u_b * u_b);
			y1_u_phase = (((atan(y1_b / y1_a) + M_PI * (y1_a < 0)) -(atan(u_b / u_a) + M_PI * (u_a < 0))) * 180.0 / M_PI);


			fprintf(stderr, "%f[Hz] %f %f[deg] %f %f[deg]\n", f, y0_u_gain, y0_u_phase, y1_u_gain, y1_u_phase);
			
			fprintf(fp[msg->spk_num], "%f %e %f %e %f\n", f, y0_u_gain, y0_u_phase, y1_u_gain, y1_u_phase);

			f10 += F10_STEP;
		}
#endif
	}

	rtai_free(nam2num("data"), data);
	rtai_free(nam2num("msg"), msg);

	fclose(fp[0]);
	fclose(fp[1]);
	fclose(fp[2]);

	return 0;
}
